@@ -1067,7 +1067,7 @@ QSet<QString> Book::GetWordsInHTMLFiles()
         all_words.append(result);
     }
 
-    return all_words.toSet();
+    return all_words.toSet();  // Qt 5.15:  QSet<QString>(all_words.begin(), all_words.end());
 }
 
 QStringList Book::GetWordsInHTMLFileMapped(HTMLResource *html_resource)
@@ -1271,7 +1271,8 @@ std::tuple<bool, QString, QString> Book::HasUndefinedURLFragments()
 {
     QList<HTMLResource *> html_resources = GetHTMLResources();
     bool hasUndefinedUrlFrags = false;
-
+    Q_UNUSED(hasUndefinedUrlFrags);
+    
     QStringList html_bookpaths;
     foreach(HTMLResource *html_resource, html_resources) {
         html_bookpaths.append(html_resource->GetRelativePath());
