@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2016-2019  Kevin B. Hendricks, Stratford, ON
+**  Copyright (C) 2016-2020  Kevin B. Hendricks, Stratford, ON
 **  Copyright (C) 2016-2020  Doug Massay
 **  Copyright (C) 2011-2013  John Schember <john@nachtimwald.com>
 **  Copyright (C) 2012-2013  Dave Heiland
@@ -39,6 +39,7 @@ static QString KEY_DEFAULT_METADATA_LANGUAGE = SETTINGS_GROUP + "/" + "default_m
 static QString KEY_UI_LANGUAGE = SETTINGS_GROUP + "/" + "ui_language";
 static QString KEY_UI_FONT = SETTINGS_GROUP + "/" + "ui_font";
 static QString KEY_ORIGINAL_UI_FONT = SETTINGS_GROUP + "/" + "original_ui_font";
+static QString KEY_DRAG_DISTANCE_TWEAK = SETTINGS_GROUP + "/" + "drag_distance_tweak";
 static QString KEY_ZOOM_IMAGE = SETTINGS_GROUP + "/" + "zoom_image";
 static QString KEY_ZOOM_TEXT = SETTINGS_GROUP + "/" + "zoom_text";
 static QString KEY_ZOOM_WEB = SETTINGS_GROUP + "/" + "zoom_web";
@@ -157,6 +158,12 @@ QString SettingsStore::originalUIFont()
     return value(KEY_ORIGINAL_UI_FONT, "").toString();
 }
 
+int SettingsStore::uiDragDistanceTweak()
+{
+    clearSettingsGroup();
+    return value(KEY_DRAG_DISTANCE_TWEAK, 0).toInt();
+}
+
 QString SettingsStore::defaultMetadataLang()
 {
     clearSettingsGroup();
@@ -473,6 +480,12 @@ void SettingsStore::setOriginalUIFont(const QString &font_data)
     setValue(KEY_ORIGINAL_UI_FONT, font_data);
 }
 
+void SettingsStore::setUiDragDistanceTweak(int tweak)
+{
+    clearSettingsGroup();
+    setValue(KEY_DRAG_DISTANCE_TWEAK, tweak);
+}
+
 void SettingsStore::setExternalXEditorPath(const QString &path)
 {
     clearSettingsGroup();
@@ -790,6 +803,7 @@ void SettingsStore::clearAppearanceSettings()
     remove(KEY_SHOWFULLPATH_ON);
     remove(KEY_HIGHDPI_SETTING);
     remove(KEY_UI_FONT);
+    remove(KEY_DRAG_DISTANCE_TWEAK);
     remove(KEY_PREVIEW_DARK_IN_DM);
     ;
 }
