@@ -26,6 +26,7 @@
 #endif
 
 #include "unzip.h"
+
 #ifdef _WIN32
 #include "iowin32.h"
 #endif
@@ -296,27 +297,6 @@ QString Utility::ReplaceFirst(const QString &before, const QString &after, const
 }
 
 
-QStringList Utility::GetAbsolutePathsToFolderDescendantFiles(const QString &fullfolderpath)
-{
-    QDir folder(fullfolderpath);
-    QStringList files;
-    foreach(QFileInfo file, folder.entryInfoList()) {
-        if ((file.fileName() != ".") && (file.fileName() != "..")) {
-            // If it's a file, add it to the list
-            if (file.isFile()) {
-                files.append(Utility::URLEncodePath(file.absoluteFilePath()));
-            }
-            // Else it's a directory, so
-            // we add all files from that dir
-            else {
-                files.append(GetAbsolutePathsToFolderDescendantFiles(file.absoluteFilePath()));
-            }
-        }
-    }
-    return files;
-}
-
-
 // Copies every file and folder in the source folder
 // to the destination folder; the paths to the folders are submitted;
 // the destination folder needs to be created in advance
@@ -324,6 +304,11 @@ void Utility::CopyFiles(const QString &fullfolderpath_source, const QString &ful
 {
     QDir folder_source(fullfolderpath_source);
     QDir folder_destination(fullfolderpath_destination);
+    folder_source.setFilter(QDir::AllDirs |
+                            QDir::Files |
+                            QDir::NoDotAndDotDot |
+                            QDir::NoSymLinks |
+                            QDir::Hidden);
     // Erase all the files in this folder
     foreach(QFileInfo file, folder_source.entryInfoList()) {
         if ((file.fileName() != ".") && (file.fileName() != "..")) {
@@ -365,7 +350,7 @@ bool Utility::removeDir(const QString &dirName)
             if (info.isDir()) {
                 result = removeDir(info.absoluteFilePath());
             } else {
-                result = QFile::remove(info.absoluteFilePath());
+                result = SDeleteFile(info.absoluteFilePath());
             }
 
             if (!result) {
@@ -374,7 +359,6 @@ bool Utility::removeDir(const QString &dirName)
         }
         result = dir.rmdir(dirName);
     }
-
     return result;
 }
 
@@ -707,7 +691,6 @@ float Utility::RoundToOneDecimal(float number)
 QWidget *Utility::GetMainWindow()
 {
     QWidget *parent_window = QApplication::activeWindow();
-
     while (parent_window && !(qobject_cast<QMainWindow *>(parent_window))) {
         parent_window = parent_window->parentWidget();
     }
@@ -715,6 +698,7 @@ QWidget *Utility::GetMainWindow()
     return parent_window;
 }
 
+
 QString Utility::getSpellingSafeText(const QString &raw_text)
 {
     // There is currently a problem with Hunspell if we attempt to pass
@@ -736,6 +720,18 @@ bool Utility::has_non_ascii_chars(const QString &str)
 
 bool Utility::use_filename_warning(const QString &filename)
 {
+    const QString uri_delimiters = ":/?#[]@";
+    foreach(QChar c, uri_delimiters) {
+        if (filename.contains(c)) {
+            QMessageBox::warning(QApplication::activeWindow(),
+                   tr("Sigil"),
+                               tr("The requested file name contains one or more url/uri delimiter characters: "
+                  " : / ? # [ ] @  "
+                  "that should NOT appear in url/uri link targets such as filenames.\n\n"),
+                   QMessageBox::Ok);
+            return false;
+        }
+    }
     if (has_non_ascii_chars(filename)) {
         return QMessageBox::Apply == QMessageBox::warning(QApplication::activeWindow(),
                 tr("Sigil"),
@@ -801,42 +797,42 @@ bool Utility::UnZip(const QString &zippath, const QString &destpath)
             // If there is no file name then we can't do anything with it.
             if (!qfile_name.isEmpty()) {
 
-	        // for security reasons against maliciously crafted zip archives
-	        // we need the file path to always be inside the target folder 
-	        // and not outside, so we will remove all illegal backslashes
-	        // and all relative upward paths segments "/../" from the zip's local 
-	        // file name/path before prepending the target folder to create 
-	        // the final path
+                // for security reasons against maliciously crafted zip archives
+                // we need the file path to always be inside the target folder 
+                // and not outside, so we will remove all illegal backslashes
+                // and all relative upward paths segments "/../" from the zip's local 
+                // file name/path before prepending the target folder to create 
+                // the final path
 
-	        QString original_path = qfile_name;
-	        bool evil_or_corrupt_epub = false;
+                QString original_path = qfile_name;
+                bool evil_or_corrupt_epub = false;
 
-	        if (qfile_name.contains("\\")) evil_or_corrupt_epub = true; 
-	        qfile_name = "/" + qfile_name.replace("\\","");
+                if (qfile_name.contains("\\")) evil_or_corrupt_epub = true; 
+                qfile_name = "/" + qfile_name.replace("\\","");
 
-	        if (qfile_name.contains("/../")) evil_or_corrupt_epub = true;
-	        qfile_name = qfile_name.replace("/../","/");
+                if (qfile_name.contains("/../")) evil_or_corrupt_epub = true;
+                qfile_name = qfile_name.replace("/../","/");
+    
+                while(qfile_name.startsWith("/")) { 
+                    qfile_name = qfile_name.remove(0,1);
+                }
+                    
+                if (cp437_file_name.contains("\\")) evil_or_corrupt_epub = true; 
+                cp437_file_name = "/" + cp437_file_name.replace("\\","");
 
-	        while(qfile_name.startsWith("/")) { 
-		  qfile_name = qfile_name.remove(0,1);
-	        }
-                
-	        if (cp437_file_name.contains("\\")) evil_or_corrupt_epub = true; 
-	        cp437_file_name = "/" + cp437_file_name.replace("\\","");
+                if (cp437_file_name.contains("/../")) evil_or_corrupt_epub = true;
+                cp437_file_name = cp437_file_name.replace("/../","/");
+    
+                while(cp437_file_name.startsWith("/")) { 
+                  cp437_file_name = cp437_file_name.remove(0,1);
+                }
 
-	        if (cp437_file_name.contains("/../")) evil_or_corrupt_epub = true;
-	        cp437_file_name = cp437_file_name.replace("/../","/");
-
-	        while(cp437_file_name.startsWith("/")) { 
-		  cp437_file_name = cp437_file_name.remove(0,1);
-	        }
-
-	        if (evil_or_corrupt_epub) {
-		    unzCloseCurrentFile(zfile);
-		    unzClose(zfile);
-		    // throw (UNZIPLoadParseError(QString(QObject::tr("Possible evil or corrupt zip file name: %1")).arg(original_path).toStdString()));
+                if (evil_or_corrupt_epub) {
+                    unzCloseCurrentFile(zfile);
+                    unzClose(zfile);
+                    // throw (UNZIPLoadParseError(QString(QObject::tr("Possible evil or corrupt zip file name: %1")).arg(original_path).toStdString()));
                     return false;
-	        }
+                }
 
                 // We use the dir object to create the path in the temporary directory.
                 // Unfortunately, we need a dir ojbect to do this as it's not a static function.
@@ -849,7 +845,7 @@ bool Utility::UnZip(const QString &zippath, const QString &destpath)
                     dir.mkpath(qfile_name);
                     continue;
                 } else {
-		    if (!qfile_info.path().isEmpty()) dir.mkpath(qfile_info.path());
+                    if (!qfile_info.path().isEmpty()) dir.mkpath(qfile_info.path());
                 }
 
                 // Open the file entry in the archive for reading.
@@ -1031,10 +1027,10 @@ QString Utility::resolveRelativeSegmentsInFilePath(const QString& file_path, con
         if (segs.at(i) == ".") continue;
         if (segs.at(i) == "..") {
             if (!res.isEmpty()) {
-	        res.removeLast();
+                res.removeLast();
             } else {
-	        qDebug() << "Error resolving relative path segments";
-		qDebug() << "original file path: " << file_path;
+                qDebug() << "Error resolving relative path segments";
+                qDebug() << "original file path: " << file_path;
             }
         } else {
             res << segs.at(i);
@@ -1152,21 +1148,22 @@ QStringList Utility::sortByCounts(const QStringList &folderlst, const QList<int>
     }
     std::sort(vec.begin(), vec.end(), sort_pair_in_reverse);
     QStringList sortedlst;
-    for(int j=0; j < vec.size(); j++) {
+    for(unsigned int j=0; j < vec.size(); j++) {
         sortedlst << vec[j].second;
     }
     return sortedlst;
 }
 
-QStringList Utility::LocaleAwareSort(QStringList &names)
+QStringList Utility::LocaleAwareSort(const QStringList &names)
 {
-  SettingsStore ss;
-  QLocale uiLocale(ss.uiLanguage());
-  QCollator uiCollator(uiLocale);
-  uiCollator.setCaseSensitivity(Qt::CaseInsensitive);
-  // use uiCollator.compare(s1, s2)
-  std::sort(names.begin(), names.end(), uiCollator);
-  return names;
+    SettingsStore ss;
+    QStringList nlist(names);
+    QLocale uiLocale(ss.uiLanguage());
+    QCollator uiCollator(uiLocale);
+    uiCollator.setCaseSensitivity(Qt::CaseInsensitive);
+    // use uiCollator.compare(s1, s2)
+    std::sort(nlist.begin(), nlist.end(), uiCollator);
+    return nlist;
 }
 
 
