@@ -31,6 +31,7 @@
 #include "ResourceObjects/MiscTextResource.h"
 #include "ResourceObjects/SVGResource.h"
 #include "Tabs/AVTab.h"
+#include "Tabs/FontTab.h"
 #include "Tabs/CSSTab.h"
 #include "Tabs/FlowTab.h"
 #include "Tabs/ImageTab.h"
@@ -589,6 +590,13 @@ bool TabManager::SwitchedToExistingTab(const Resource *resource,
             return true;
         }
 
+        FontTab *font_tab = qobject_cast<FontTab *>(tab);
+
+        if (font_tab != NULL) {
+            return true;
+        }
+
+
     }
 
     return false;
@@ -670,6 +678,11 @@ ContentTab *TabManager::CreateTabForResource(Resource *resource,
             break;
         }
 
+        case Resource::FontResourceType: {
+            tab = new FontTab(qobject_cast<Resource *>(resource), this);
+            break;
+        }
+
         default:
             break;
     }
