@@ -126,6 +126,9 @@ extern const QString PYTHON_MAIN_PATH;
 extern const QString PYTHON_MAIN_BIN_PATH;
 #if defined(__APPLE__)
 extern const QString PYTHON_SITE_PACKAGES;
+extern const QString PYTHON_MAIN_PREFIX;
+extern const QString PYTHON_MAIN_BIN_PATH;
+extern const QString PYTHON_LIB_PATH;
 #endif
 extern const QStringList PYTHON_SYS_PATHS;
 
